from functools import partial
import random
import string
from itertools import cycle
from django_dynamic_fixture import G
import uuidfield
from sample_data_utils.sample import text
from sample_data_utils.utils import sequence
#from django_any import any_field, any_model
#from django_any.contrib import any_model_with_defaults
from geo.models import Currency, Country, AdministrativeAreaType, AdministrativeArea, LocationType, Location


nextname = partial(sequence, cache={})

# Create two-character iso-codes
iso2_codes = []
for c1 in string.ascii_uppercase:
    for c2 in string.ascii_uppercase:
        iso2_codes.append('{}{}'.format(c1, c2))
random.shuffle(iso2_codes)
iso2_codes_iter = cycle(iso2_codes)

# Create three-character iso-codes
iso3_codes = []
for c1 in string.ascii_uppercase:
    for c2 in string.ascii_uppercase:
        for c3 in string.ascii_uppercase:
            iso3_codes.append('{}{}{}'.format(c1, c2, c3))
random.shuffle(iso3_codes)
iso3_codes_iter = cycle(iso3_codes)

# Create iso-number with three digits
iso_numbers = range(100, 999)
random.shuffle(iso_numbers)
iso_numbers_iter = cycle(iso_numbers)

#
#@any_field.register(uuidfield.UUIDField)
#def uuid_field(field, **kwargs):
#    return uuidfield.UUIDField()._create_uuid()


def subargs(kwargs, prefix):
    prefix = "%s__" % prefix
    ret = {key[len(prefix):]: kwargs.pop(key) for key in kwargs.keys() if key.startswith(prefix)}
    return ret


def currency_factory(**kwargs):
    return G(Currency, code=nextname('CUR').next())


def country_factory(**kwargs):
    iso_code = iso2_codes_iter.next()
    currency = G(Currency, code=nextname(iso_code))

    kwargs.setdefault('iso_code', iso_code)
    kwargs.setdefault('num_code', iso_numbers_iter.next())
    kwargs.setdefault('iso3_code', iso3_codes_iter.next())
    kwargs.setdefault('name', nextname('Country'))
    kwargs.setdefault('fullname', text(20))
    kwargs.setdefault('currency', currency)

    country = G(Country, **kwargs)
    return country


def area_tree_factory(country):
    parent = area_factory(country, name=nextname('RegionArea'), type__name=nextname('Region'))
    area = area_factory(parent=parent, name=nextname('SubRegionArea'), type__name=nextname('SubRegion'))
    location = location_factory(area=area)
    return parent, area, location


def area_type_factory(country, **kwargs):
    kwargs.setdefault('name', nextname('AdministrativeAreaType'))
    kwargs.setdefault('parent', None)
    return AdministrativeAreaType.objects.get_or_create(country=country, **kwargs)[0]


def area_factory(country=None, **kwargs):
    if 'parent' in kwargs:
        country = kwargs['parent'].country
        type_kwargs = subargs(kwargs, 'type')
        type_kwargs['country'] = country
        type_kwargs['parent'] = kwargs['parent'].type
        kwargs['type'] = area_type_factory(**type_kwargs)
    else:
        type_kwargs = subargs(kwargs, 'type')
        type_kwargs['country'] = country
        kwargs['type'] = area_type_factory(**type_kwargs)

    kwargs.setdefault('parent', None)
    kwargs.setdefault('name', nextname('AdministrativeArea'))

    return AdministrativeArea.objects.get_or_create(country=country, **kwargs)[0]


def location_type_factory(**kwargs):
    return G(LocationType, **kwargs)


def location_factory(**kwargs):
    if 'area' in kwargs:
        kwargs['country'] = kwargs['area'].country
    else:
        if not 'country' in kwargs:
            kwargs['country'] = country_factory(**subargs(kwargs, 'country'))
        area_kwargs = subargs(kwargs, 'area')
        area_kwargs['country'] = kwargs['country']
        kwargs['area'] = area_factory(**area_kwargs)
    if 'type' not in kwargs:
        kwargs['type'] = location_type_factory(**subargs(kwargs, 'type'))

    kwargs.setdefault('name', nextname('Location'))
    location = G(Location, **kwargs)
    return location



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'cv.sqlite', # Or path to database file if using sqlite3.
        'USER': '', # Not used with sqlite3.
        'PASSWORD': '', # Not used with sqlite3.
        'HOST': '', # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '', # Set to empty string for default. Not used with sqlite3.
    }
}
INSTALLED_APPS = (
    'geo',
)
SECRET_KEY = '123'

DDF_VALIDATE_ARGS = True
DDF_DEBUG_MODE = True  # default = False
DDF_IGNORE_FIELDS = ['version', 'last_modified_user', 'last_modified_date', 'security_hash']
DDF_FILL_NULLABLE_FIELDS = True
DDF_DEFAULT_DATA_FIXTURE = 'geo.tests.ddf_fixture.DataFixture'  # static_sequential, random, sequential,



# -*- coding: utf-8 -*-
import StringIO
import unicodecsv as csv
from functools import wraps
import os
import zipfile
from django.core.exceptions import ValidationError
from django.core.management.base import BaseCommand, CommandError
from optparse import make_option
from django.db import transaction, IntegrityError
from geo.models import Country, Currency, CONTINENTS
import requests

DATA_DIR = os.path.join(os.path.dirname(__file__), 'DATADIR')
if not os.path.exists(DATA_DIR):
    os.makedirs(DATA_DIR)


def cache(func):
    def _inner(filename):
        if os.path.exists(os.path.join(DATA_DIR, filename)):
            with open(os.path.join(DATA_DIR, filename), 'r') as f:
                content = f.read()
        else:
            content = func(filename)
            with open(os.path.join(DATA_DIR, filename), 'wb') as f:
                f.write(content)
        return content

    return _inner


def geoload(filename):
    url = 'http://download.geonames.org/export/dump/%s' % filename

    if os.path.exists(os.path.join(DATA_DIR, filename)):
        with open(os.path.join(DATA_DIR, filename), 'r') as f:
            content = f.read()
    else:
        r = requests.get(url)
        assert r.status_code == 200, r.status_code
        assert r.headers['content-type'] != 'text/html; charset=utf-8', r.headers['content-type']
        with open(os.path.join(DATA_DIR, filename), 'wb') as f:
            f.write(r.content)
        content = r.content

    name, ext = filename.rsplit('.', 1)
    if ext == 'zip':
        z = zipfile.ZipFile(StringIO.StringIO(content))
        return z.read(name + '.txt')

    return content


class Command(BaseCommand):
    args = ''
    option_list = BaseCommand.option_list + (
        make_option('-a', '--all', action='store_true', dest='all'),
        make_option('-c', '--country', action='store_true', dest='country'),
        #make_option('-t', '--city', action='store_true', dest='city'),
        make_option('-z', '--timezone', action='store_true', dest='tz'),
        make_option('-m', '--currency', action='store_true', dest='currency'),
        make_option('-r', '--region', action='store_true', dest='region'),
        make_option('-i', '--ignore-cache', action='store_true', dest='reset'),
    )

    def _split(self, text, sep='\t', linesep='\n', ignore='#'):
        lines = []
        for line in text.split(linesep):
            if line.startswith(ignore):
                continue
            elif not line.strip():
                continue
            lines.append(line.split(sep))
        return lines

    def handle(self, *args, **options):
        load_all = options.get('all')
        reset = options.get('currency')
        if load_all:
            region = country = city = tz = currency = True
        else:
            country = options.get('country')
            region = options.get('region')
            city = options.get('city')
            tz = options.get('timezone')
            currency = options.get('currency')

        if region:
            self.regions()
        if country:
            self.countries()

    #def cities(self):
    #    data = geoload('cities15000.zip')

    def countries(self):
        data = geoload('countryInfo.txt')
        # data = geoload('allCountries.zip')
        # data = load('IT.zip')
        lines = self._split(data)
        transaction.enter_transaction_management(True)
        for line in lines:
            if line:
                try:
                    iso2, iso3, ison, fips, name, \
                    capital, area, population, \
                    continent, tld, \
                    currencycode, currencyname, \
                    phone, postal_code_format, \
                    postal_code_regex, \
                    languages, \
                    geonameid, neighbours, \
                    equivalentfipscode = line

                    c, __ = Country.objects.get_or_create(num_code=ison,
                                                          defaults={'iso3_code': iso3,
                                                                    'iso_code': iso2,
                                                                    'name': name,
                                                                    'fullname': name})
                    c.tld = tld
                    c.geonameid = geonameid or None
                    c.continent = continent.upper()

                    curr, __ = Currency.objects.get_or_create(code=currencycode,
                                                                defaults={'name': currencyname})
                    c.currency = curr
                    c.clean()
                    c.save()
                except Country.DoesNotExist:
                    print geonameid, line
                except (ValueError, IntegrityError, ValidationError) as e:
                    print e, line
                    transaction.rollback()
                    raise
                except:
                    transaction.rollback()
                    raise
                else:
                    transaction.commit()
